import { world, system, Player, ItemStack, EffectTypes } from '@minecraft/server';
import { Global } from '../Global.js';
import { MapUtil, LoopUtil, ItemUtil, FirearmUtil } from "../Utilities.js";
import { AnimationLink } from '../AnimationLink.js';

import { shoot } from "../Shoot.js";
import { FiringModes, GunWithAbility } from '../Definitions/FirearmDefinition.js';
import { SwitchFiringModeAttributes } from '../Definitions/LeftClickAbilityDefinition.js';


//Adds new players to the shootingID dictionary
world.afterEvents.playerSpawn.subscribe((eventData) => {
    const player = eventData.player;
    if(eventData.initialSpawn) {
        Global.playerShootingLoopIds.set(player.id, []);
        //MapUtil.printMap(Global.playerShootingLoopIds);
    }
});

//Deletes dictionary values for players who leave
world.beforeEvents.playerLeave.subscribe((eventData) => {
    const player = eventData.player;
    system.run(() => {
        Global.playerShootingLoopIds.delete(player.id);
        //MapUtil.printMap(Global.playerShootingLoopIds);
    });
});


/**
 * @param {Player} player 
 * @param {ItemStack} itemStack 
 */
function shootDetection(player, itemStack) {
    if(!FirearmUtil.isHoldingFirearm(player)) { return; }
    const firearmObject =  FirearmUtil.getFirearmObjectFromItemStack(itemStack);
    if(firearmObject === null) { return; }
    if(!FirearmUtil.isOffhandAnAmmoType(player))              { console.log("no ammo in offhand"); return; }
    if(!FirearmUtil.isOffhandAmmoTypeCorrect(player))         { console.log("wrong ammo type"); return; }
    if((FirearmUtil.getAmmoCountFromOffhand(player)??0) <= 0) { console.log("out of ammo"); return; }
    /*
    player.setDynamicProperty(Global.PlayerDynamicProperties.script.isFirstShot, true);

    if(firearmObject instanceof GunWithAbility && firearmObject.leftClickAbilityAttributes instanceof SwitchFiringModeAttributes) {
        if(itemStack.getDynamicProperty(Global.ItemAbilityDynamicProperties.currentFiringMode) === FiringModes.auto || (itemStack.getDynamicProperty(Global.ItemAbilityDynamicProperties.currentFiringMode) === undefined && firearmObject.firingMode === FiringModes.auto)) {
            //LoopUtil.startAsyncLoop(player, Global.playerShootingLoopIds, function(){ return shoot(player, firearmObject); }, firearmObject.firingRate);
        }
        else if(itemStack.getDynamicProperty(Global.ItemAbilityDynamicProperties.currentFiringMode) === FiringModes.semi || (itemStack.getDynamicProperty(Global.ItemAbilityDynamicProperties.currentFiringMode) === undefined && firearmObject.firingMode === FiringModes.semi)) {
            //shoot(player, firearmObject);
        }
    }
    else {
        if(firearmObject.firingMode === FiringModes.auto) {
            //LoopUtil.startAsyncLoop(player, Global.playerShootingLoopIds, function(){ return shoot(player, firearmObject); }, firearmObject.firingRate);
        }
        else if(firearmObject.firingMode === FiringModes.semi) {
            //shoot(player, firearmObject);
        }
    }
        */
    player.setDynamicProperty(Global.PlayerDynamicProperties.animation.is_shooting, true);
    AnimationLink.renewClientAnimationVariable(player, Global.PlayerDynamicProperties.animation.is_shooting);
    
    //console.log(`player ${player.name} is using item ${itemStack.typeId}`);
}

//---------------------- Ran in Main.js ----------------------
export { shootDetection };
//---------------------- Ran in Main.js ----------------------


world.afterEvents.itemStopUse.subscribe((eventData) => {
    const player    = eventData.source;
	const itemStack = eventData.itemStack;

    LoopUtil.stopAsyncLoop(player, Global.playerShootingLoopIds);
    player.setDynamicProperty(Global.PlayerDynamicProperties.animation.is_shooting, false);
    AnimationLink.renewClientAnimationVariable(player, Global.PlayerDynamicProperties.animation.is_shooting);
    if(!FirearmUtil.isHoldingFirearm(player)) { return; }
    if(itemStack === undefined) { return; }
    const firearmContainerSlot = ItemUtil.getSelectedContainerSlot(player);
    if(firearmContainerSlot !== null) { FirearmUtil.tryCopyWorldAmmoToMainhandFirearm(firearmContainerSlot); }

    
    //console.log(`player ${player.name} stopped using item ${itemStack.typeId}`);
});

world.afterEvents.entityDie.subscribe((eventData) => {
    if(!(eventData.deadEntity instanceof Player)) { return; }
    const player = eventData.deadEntity;
    LoopUtil.stopAsyncLoop(player, Global.playerShootingLoopIds);
});