class Global {
    /**
     * A dictionary of all guns
     * `key: FirearmTags enum {string}`, `value: Firearm Object {Firearm}`
     */
    static firearms = new Map();

    /**
     * A dictionary of all guns
     * `key: MagazineTags enum {string}`, `value: Magazine Object {Magazine}`
     */
    static magazines = new Map();

    /**
     * A dictionary to store mainLoopIds and functions
     * `key: mainLoopId {number}`, `value: function()`
     */
    static mainLoops = new Map();

    /**
     * A dictionary to store shooting loop ids
     * `key: player.id {string}`, `value: loopId {number}`
     */
    static playerShootingLoopIds = new Map();

    /**
     * A dictionary to store reloading loop ids
     * `key: player.id {string}`, `value: loopId {number}`
     */
    static playerReloadingLoopIds = new Map();

    /**
     * A dictionary to store left click ability loop ids
     * `key: player.id {string}`, `value: loopId {number}`
     */
    static playerFirearmAbilityLoopIds = new Map();
    /**
     * A dictionary to store left click ability entities
     * `key: player.id {string}`, `value: leftClickAbilityEntity {Entity}`
     */
    static playerFirearmAbilityEntities = new Map();


    /**
     * A dictionary to store the firearm id of firearms that players are currently holding.
     * If they aren't holding a firearm, then tey are not in the map.
     * `key: player.id {string}`, `value: firearmId {number}`
     */
    static playerCurrentFirearmId = new Map();

    /**
     * A dictionary to store the firearm itemstacks that players are currently holding.
     * If they aren't holding a firearm, then tey are not in the map.
     * Used to check item dynamic properties when the player switches to a different item.
     * `key: player.id {string}`, `value: firearmItemStack {ItemStack}`
     */
    static playerCurrentFirearmItemStack = new Map();


    /**
     * A list of maps containing all firearmIds and their ammo count
     * Map(`key: firearm id {number}`, `value: ammoCount {number}`)
     * Is initialized in WorldInitialization.js
    */
    //static worldFirearmIds = [];


    /**
     * An enum of all player dynamic properties
     */
    static PlayerDynamicProperties = {
        "script": {
            isInThirdPersonCamera:        "isInThirdPersonCamera",        //boolean reset across reload
            loadedOffhandMagazine:        "loadedOffhandMagazine",        //boolean keep across reload
            currentFirearmIdSaved:        "currentFirearmIdSaved",        //boolean reset across reload
            currentFirearmItemStackSaved: "currentFirearmItemStackSaved", //boolean reset across reload
            isAiming:                     "isAiming",                     //boolean reset across reload
            isHoldingAbilityFirearm:      "isHoldingAbilityFirearm"       //boolean reset across reload
        },

        "animation": {
            team:                       "team",                        //Team enum ("gold" or "blue"), reset across reload
            has_offhand_magazine:       "has_offhand_magazine",        //boolean reset across reload
            firearm_firing_mode:        "firearm_firing_mode",         //boolean reset across reload
            firearm_fire_rate:          "firearm_fire_rate",           //boolean reset across reload
            firearm_has_ammo:           "firearm_has_ammo",            //boolean reset across reload
            should_cock_on_reload:      "should_cock_on_reload",       //boolean reset across reload
            is_shooting:                "is_shooting",                 //boolean reset across reload
            is_reloading:               "is_reloading",                //boolean reset across reload
            should_start_cock:          "should_start_cock",           //boolean reset across reload
            recoil:                     "recoil",                      //float reset across reload
            movement_direction:         "movement_direction",          //Direction enum {string} reset across reload
            movement_direction_value:   "movement_direction_value",    //float reset across reload
            reload_normal_animation_multiplier: "reload_normal_animation_multiplier",   //float reset across reload
            reload_no_swap_animation_multiplier: "reload_no_swap_animation_multiplier", //float reset across reload
            reload_cock_animation_multiplier: "reload_cock_animation_multiplier",        //float reset across reload

            view_direction_x: "view_direction_x", //float reset across reload
            view_direction_y: "view_direction_y", //float reset across reload
            view_direction_z: "view_direction_z"  //float reset across reload
        }
    }

    //Only used in worldInitialization
    static PersistentPlayerDynamicProperties = {
        /**
         * Must keep `loadedOffhandMagazine` or else the player will get a duplicate magazine every time they join
         * because the game will think the magazine in their hand was not a part of the firearm.
         */
        loadedOffhandMagazine: "loadedOffhandMagazine",
        /** These are tied to loadedOffhandMagazine from HoldDetection() */
        has_offhand_magazine:  "has_offhand_magazine",
        should_cock_on_reload: "should_cock_on_reload",
        firearm_has_ammo:           "firearm_has_ammo"

    }

    /**
     * An enum of all item dynamic properties
     */
    static ItemDynamicProperties = {
        id: "id",                          //number
        /**
         * Is a less accurate rep of ammoCount compared to world ammoCount, but is used after world reload.
         * World ammoCount is deleted after world reload, which takes the firearm ammoCount when requested.
         * Takes the firearm id's world ammoCount when:
         *   1. The player holds the firearm (first tick)
         *   2. When the player stops shooting
         */
        ammoCount: "ammoCount",            //number {}
        magazineTag: "magazineTag",        //MagazineTags enum {string}
        isMagazineEmpty: "isMagazineEmpty" //boolean
    }

    /**
     * An enum of all firearm ability dynamic properties
     */
    static ItemAbilityDynamicProperties = {
        currentScopeZoom: "currentScopeZoom",      //number, can either be 1 or 2, for switchScopeZoom LeftClickAbility 
        currentFiringMode: "currentFiringMode",    //string, FiringMode enum, for switchFiringMode LeftClickAbility
    }
}

export { Global };