import "./CustomComponents/Initialization.js";
import "./Lists/InitializeScoreboards.js";
import "./WorldInitialization.js";

import "./Lists/FirearmsList.js";
import "./Lists/MagazinesList.js";

import "./ScriptEvent/Functions.js";
import "./ScriptEvent/FirearmScriptEvents.js";

//import "./Detectors/ThirdPersonDetection.js";


import { world, system, EffectTypes, ItemStack, EntityComponentTypes, EntityRideableComponent } from '@minecraft/server';
import { Vector3 } from './Math/Vector3.js';
import { Global } from "./Global.js";
const Vector = new Vector3();


import * as ShootDetection from "./Detectors/ShootDetection.js";
import * as AutoReloadDetection from "./Detectors/AutoReloadDetection.js";


world.afterEvents.itemStartUse.subscribe((eventData) => {
    const player = eventData.source;
    const itemStack = eventData.itemStack;
    ShootDetection.shootDetection(player, itemStack);
    FirearmUtil.renewFirearmAmmoOnMagazineChange(player);
    AutoReloadDetection.automaticReloadDetection(player, itemStack, false);

    //flashlightTest(player, itemStack);
});


import * as HoldDetection from "./Detectors/HoldDetection.js";
import * as DirectionDetection from "./Detectors/DirectionDetection.js";
import * as OffhandStackCheck from "./Detectors/OffhandStackDetection.js";
import * as AimDetection from "./Detectors/AimDetection.js";
import { renewAmmoCount } from "./AmmoText.js";
import { FirearmUtil } from "./Utilities.js";
import { AnimationLink } from "./AnimationLink.js";

system.runInterval(() => {
    world.getAllPlayers().forEach(player => {
        DirectionDetection.directionDetection(player);
        OffhandStackCheck.offhandStackCheck(player);
        HoldDetection.holdingFirearmDetectionPart1(player);
    });
    for(const func of Global.mainLoops.values()) {
        func(); //runs all main loop functions here
    }
    world.getAllPlayers().forEach(player => {
        HoldDetection.holdingFirearmDetectionPart2(player);
        if(system.currentTick % 15 === 0) { renewAmmoCount(player); }

        if(player.getDynamicProperty(Global.PlayerDynamicProperties.animation.is_shooting) === false) {
            FirearmUtil.tryDecreaseRecoil(player);
        }
        AimDetection.aimDetection(player);
        FirearmUtil.renewFirearmAmmoOnMagazineChange(player);
        //LeftClickAbilityDetection.leftClickAbilityDetection(player);

        //tempoarary placement
        if(system.currentTick % 5 === 0) {
            const viewDir = player.getViewDirection();
            player.setDynamicProperty(Global.PlayerDynamicProperties.animation.view_direction_x, viewDir.x);
            player.setDynamicProperty(Global.PlayerDynamicProperties.animation.view_direction_y, viewDir.y);
            player.setDynamicProperty(Global.PlayerDynamicProperties.animation.view_direction_z, viewDir.z);
            AnimationLink.renewClientAnimationVariable(player, Global.PlayerDynamicProperties.animation.view_direction_x);
            AnimationLink.renewClientAnimationVariable(player, Global.PlayerDynamicProperties.animation.view_direction_y);
            AnimationLink.renewClientAnimationVariable(player, Global.PlayerDynamicProperties.animation.view_direction_z);
        }
    });
});

//system.runInterval(() => {
//    world.setDynamicProperty("firearmId:"+getRandomNumber(Math.pow(2,-32),Math.pow(2,32)).toString(), 999);
//},0);

/**
 * |---------------------------------------------------------|
 * |                                                         |
 * | Make sure all firearms have the "yes:is_firearmgun" tag |
 * |     and the tag of their own variant like "yes:ak47"    |
 * |                                                         |
 * |---------------------------------------------------------|
 * |                                                         |
 * |  Make sure all magazines have the "yes:is_magazine" tag |
 * |  and of their own variant like "yes:rifle_magazine_30"  |
 * |                                                         |
 * |---------------------------------------------------------|
 */

